package poi.test.code;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;
import java.nio.channels.FileChannel;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import junit.framework.TestCase;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.openxml4j.opc.PackagePart;
import org.apache.poi.openxml4j.opc.PackagePartName;
import org.apache.poi.openxml4j.opc.PackagingURIHelper;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class WriteIntoPackagePart extends TestCase {

	String strCommentOriginal;
	String strCommentNewLoaded;
	String strCommentAfterManipulation;
	PackagePart commentOriginal;

	String path = "F:\\DeletePartComment.docx";
	String newDocumentName = "output.docx";
	String newPath = "F:\\" + newDocumentName;

	@Before
	protected void setUp() throws Exception {
		File inF = new File(path); 
        File outF = new File(path+"t"); 
        copyFile(inF, outF); 

	}

	@After
	protected void tearDown() throws Exception {
		File inF = new File(path+"t");
        File outF = new File(path);  
		
        copyFile(inF, outF); 
	}

	@Test
	public void testOriginalCommentInputStreamContainsData() {
		OPCPackage mainPackage;

		DocumentBuilder builder;
		try {
			mainPackage = OPCPackage.open(path);

			PackagePartName commentsPart = PackagingURIHelper
					.createPartName(WordXMLConstants.WORD_COMMENT_PART_NAME);
			commentOriginal = mainPackage.getPart(commentsPart);

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setIgnoringElementContentWhitespace(true);
			factory.setNamespaceAware(true);

			builder = factory.newDocumentBuilder();
			Document domComment = builder.parse(commentOriginal
					.getInputStream());
			domComment.setXmlStandalone(true);
			domComment.setXmlVersion("1.0");

			StringWriter output = new StringWriter();

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();
			transformer.transform(new DOMSource(domComment), new StreamResult(
					output));

			strCommentOriginal = output.toString();
			System.out.println("Orginal Comment.xml:");
			System.out.println(strCommentOriginal);
			System.out.println();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		assertTrue(strCommentOriginal.length() > 0);

	}

	@Test
	public void testOrignalModifiedCommentContainsData() {
		int commentId = 299;

		OPCPackage mainPackage;
		try {
			mainPackage = OPCPackage.open(path);
			PackagePartName commentsPart = PackagingURIHelper
					.createPartName(WordXMLConstants.WORD_COMMENT_PART_NAME);
			commentOriginal = mainPackage.getPart(commentsPart);

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setIgnoringElementContentWhitespace(true);
			factory.setNamespaceAware(true);

			DocumentBuilder builder = factory.newDocumentBuilder();
			Document domComment = builder.parse(commentOriginal
					.getInputStream());
			domComment.setXmlStandalone(true);
			domComment.setXmlVersion("1.0");

			WordFinding woTag = new WordFinding("path", "rule1", "commentText",
					"pathOfFile", "0_0_0");
			for (int i = 0; i < 4; i++) {

				domComment.getFirstChild()
						.appendChild(
								Comment.getCommentWithID(domComment,
										++commentId, woTag));
			}

			removeTextNodes(domComment.getFirstChild());

			StringWriter output = new StringWriter();

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();
			transformer.transform(new DOMSource(domComment), new StreamResult(
					output));

			strCommentAfterManipulation = output.toString();
			

			strCommentOriginal = output.toString();
			System.out.println("Modified Comment.xml:");
			System.out.println(strCommentAfterManipulation);
			System.out.println();

			OutputStream mpOut = commentOriginal.getOutputStream();

			mpOut.write(strCommentAfterManipulation.getBytes(), 0,
					strCommentAfterManipulation.length());

			mainPackage.save(new File(newPath));

			mainPackage.flush();
			mainPackage.close();

		} catch (InvalidFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		assertTrue(strCommentAfterManipulation.length()>0);
	}

	@Test
	public void testLoadModifiedCommentDataContainsData() {
		OPCPackage newPackage;
		PackagePart commentPartNewLoad;
		Document domComment;
		try {
			PackagePartName commentsPart = PackagingURIHelper
					.createPartName(WordXMLConstants.WORD_COMMENT_PART_NAME);
			newPackage = OPCPackage.open(newPath);
			commentPartNewLoad = newPackage.getPart(commentsPart);

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setIgnoringElementContentWhitespace(true);
			factory.setNamespaceAware(true);

			DocumentBuilder builder = factory.newDocumentBuilder();
			domComment = builder.parse(commentPartNewLoad.getInputStream());
			domComment.setXmlStandalone(true);
			domComment.setXmlVersion("1.0");

			StringWriter output = new StringWriter();

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();
			transformer.transform(new DOMSource(domComment), new StreamResult(
					output));

			strCommentNewLoaded = output.toString();
			

			strCommentOriginal = output.toString();
			System.out.println("Modified loaded Comment.xml:");
			System.out.println(strCommentNewLoaded);
			System.out.println();

		} catch (InvalidFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassCastException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		assertTrue(strCommentNewLoaded.length()>0);
	}

	private static void removeTextNodes(Node start) {
		boolean ende = false;
		boolean newStart = false;
		NodeList allNodes = start.getChildNodes();
		Node checkNode;
		while (!ende) {
			newStart = false;
			allNodes = start.getChildNodes();
			for (int i = 0; i < allNodes.getLength(); i++) {
				checkNode = allNodes.item(i);
				if (checkNode.hasChildNodes()) {
					removeTextNodes(checkNode);
				}
				if (checkNode.getNodeType() == Node.TEXT_NODE) {
					start.removeChild(checkNode);
					newStart = true;
					break;
				}
			}
			if (!newStart) {
				ende = true;
			}
		}
	}
	
	 public static void copyFile(File in, File out) throws IOException { 
	        FileChannel inChannel = new FileInputStream(in).getChannel(); 
	        FileChannel outChannel = new FileOutputStream(out).getChannel(); 
	        try { 
	            inChannel.transferTo(0, inChannel.size(), outChannel); 
	        } catch (IOException e) { 
	            throw e; 
	        } finally { 
	            if (inChannel != null) 
	                inChannel.close(); 
	            if (outChannel != null) 
	                outChannel.close(); 
	        } 
	    } 
}
